﻿//!CompilerOption:AddRef:SharpSvn.dll
using System;
using System.IO;
using System.Linq;
using System.Reflection;
using System.Threading;
using SharpSvn;

namespace EZUpdater
{
    public class EZUpdater
    {
        public static Version PluginVersion = new Version(1, 0, 4);

        #region singleton
        public static EZUpdater _instance;

        public static EZUpdater Instance
        {
            get { return _instance ?? (_instance = new EZUpdater()); }
        }
        #endregion
        private static Thread _ezUpdaterThread;

        #region constructor
        private EZUpdater()
        {
        }
        #endregion

        internal void StartWorker()
        {
            try
            {
                if (_ezUpdaterThread == null)
                {
                    Logger.Info("Starting EZUpdater Thread");

                    _ezUpdaterThread = new Thread(EZWorker)
                    {
                        Name = "EZUpdater",
                        IsBackground = true
                    };
                    _ezUpdaterThread.Start();
                }
            }
            catch (Exception ex)
            {
                Logger.Error("Error starting EZUpdater: {0}", ex.ToString());
            }
        }

        internal void StopWorker()
        {
            try
            {
                if (_ezUpdaterThread != null && _ezUpdaterThread.IsAlive)
                {
                    _ezUpdaterThread.Abort();
                }
            }
            catch (ThreadAbortException)
            { }
            catch (Exception ex)
            {

            }
        }

        private static void EZWorker()
        {
            try
            {
                string demonbuddyPath = Path.GetDirectoryName(Assembly.GetEntryAssembly().Location);
                string workingDirectory = Path.Combine(demonbuddyPath, "EZUpdaterStorage");
                //Logger.InfoFormat("Demonbuddy path: {0}", demonbuddyPath);
                //Logger.InfoFormat("workingDirectory path: {0}", workingDirectory);
                TimeSpan updateDuration = new TimeSpan(24, 0, 0);

                string pluginBasePath = Path.Combine(demonbuddyPath, Zeta.Bot.Settings.GlobalSettings.Instance.PluginsPath);
                string profileBasePath = Path.Combine(demonbuddyPath, "Profiles");

                bool isValid = true;

                while (isValid)
                {
                    try
                    {
                        bool restartRequired = false;

                        foreach (Repository repo in EZSettings.Instance.Repositories.Where(p => p.Enabled))
                        {
                            // Validity check... reset to defaults if anything is broken!
                            if (string.IsNullOrWhiteSpace(repo.Name) ||
                                string.IsNullOrWhiteSpace(repo.RepoURL))
                            {
                                Logger.Error("Settings are INVALID. Resetting to defaults. Please restart Demonbuddy for correct settings.");
                                EZSettings.Instance.InitializeDefaults();
                                isValid = false;
                                return;
                            }

                            string workingPath, workingTargetPath, pluginPath, profilePath;

                            try
                            {
                                workingPath = Path.Combine(workingDirectory, repo.Name);
                                workingTargetPath = workingPath;

                                if (!string.IsNullOrWhiteSpace(repo.SourceDirectory))
                                    workingTargetPath = Path.Combine(workingPath, repo.SourceDirectory);

                                pluginPath = Path.Combine(pluginBasePath, repo.Name);
                                profilePath = Path.Combine(profileBasePath, repo.Name);
                            }
                            catch (Exception ex)
                            {
                                Logger.Error("Error setting directory paths for Name={0}, SourceDirectory={1} workingDirectory={2} \n{3}", repo.Name, repo.SourceDirectory, workingDirectory, ex);
                                continue;
                            }

                            bool isProfile = repo.Kind == RepoKind.Profile;

                            string destinationPath = isProfile ? profilePath : pluginPath;

                            bool needsUpdate = false;

                            if (DateTime.UtcNow.Subtract(repo.LastUpdate) > updateDuration)
                            {
                                Logger.Info("More than {0} since last update", updateDuration);
                                needsUpdate = true;
                            }

                            if (!Directory.Exists(workingPath))
                            {
                                Logger.Info("Working path {0} is missing", workingPath);
                                needsUpdate = true;
                                Directory.CreateDirectory(workingPath);
                            }

                            if (!Directory.Exists(destinationPath))
                            {
                                Logger.Info("Destination path {0} is missing", pluginPath);
                                needsUpdate = true;
                            }

                            if (!needsUpdate)
                            {
                                continue;
                            }

                            Logger.Info("Checking for Update for {0}", repo.Name);

                            repo.LastUpdate = DateTime.UtcNow;
                            EZSettings.Instance.Save();

                            //SvnUpdateResult provides info about what happened during a checkout
                            SvnUpdateResult result;

                            //SvnCheckoutArgs wraps all of the options for the 'svn checkout' function
                            SvnCheckOutArgs args = new SvnCheckOutArgs();

                            //the using statement is necessary to ensure we are freeing up resources
                            using (SvnClient client = new SvnClient())
                            {
                                try
                                {
                                    //SvnUriTarget is a wrapper class for SVN repository URIs
                                    SvnUriTarget svnUritarget = new SvnUriTarget(repo.RepoURL);

                                    var uri = client.GetUriFromWorkingCopy(workingPath);
                                    bool updateAvailable = false;
                                    if (uri != null)
                                    {
                                        SvnStatusArgs statusArgs = new SvnStatusArgs()
                                        {
                                            RetrieveRemoteStatus = true
                                        };

                                        SvnStatusEventArgs statusEvent = null;

                                        // Only check if we actually need to update
                                        client.Status(workingPath, statusArgs, delegate(object sender, SvnStatusEventArgs sea) { statusEvent = sea; });

                                        if (statusEvent != null && statusEvent.IsRemoteUpdated)
                                        {
                                            Logger.Info("Update is available for {0} at {1}", repo.Name, workingPath);
                                            updateAvailable = true;
                                        }
                                        else
                                        {
                                            Logger.Info("No update available for {0} at {1}", repo.Name, workingPath);

                                        }
                                    }
                                    else
                                    {
                                        Logger.Info("Invalid working copy for {0} at {1}", repo.Name, workingPath);
                                        updateAvailable = true;
                                    }

                                    //this is the where 'svn checkout' actually happens.
                                    if (updateAvailable && client.CheckOut(svnUritarget, workingPath, args, out result) && client.Update(workingPath))
                                    {
                                        Thread.Sleep(100);
                                        Logger.Info("Successfully checked out {0} revision {1} to {2}", repo.Name, result.Revision, workingPath);
                                        Logger.Info("Copying files from {0} to {1}", workingTargetPath, destinationPath);
                                        FileUtils.DirectoryCopy(workingTargetPath, destinationPath, true, ".svn");

                                        if (!isProfile)
                                            restartRequired = true;
                                    }
                                    if (!updateAvailable && needsUpdate)
                                    {
                                        FileUtils.DirectoryCopy(workingTargetPath, destinationPath, true, ".svn");
                                        if (!isProfile)
                                            restartRequired = true;
                                    }
                                }
                                catch (SvnException se)
                                {
                                    Logger.Error("svn error: {0}", se.ToString());

                                    if (client.CleanUp(workingPath))
                                    {
                                        Logger.Info("Cleaned up working path {0}", workingPath);
                                    }

                                }
                                catch (UriFormatException ufe)
                                {
                                    Logger.Error("svn error: {0}", ufe.ToString());
                                    if (client.CleanUp(workingPath))
                                    {
                                        Logger.Info("Cleaned up working path {0}", workingPath);
                                    }
                                }
                            }
                        }

                        if (restartRequired)
                        {
                            Logger.Info("**********************************************************************************");
                            Logger.Info("********************업데이트는 한번이면 됩니다. 데몬버디를 재부팅해주세요************************");
                            Logger.Info("**********modify by XXjinhyokXX 한국포럼분들을 위하여 만들었습니다. 무단배포 금지.**************");
                        }

                        Thread.Sleep(new TimeSpan(0, 0, 1));

                    }
                    catch (ThreadAbortException)
                    { }
                    catch (Exception ex)
                    {
                        Logger.Error("Error in EZWorker Thread: {0}", ex.ToString());
                        EZSettings.Instance.SetDefaults();
                        EZSettings.Instance.Save();
                    }
                }

            }
            catch (Exception ex)
            {
                Logger.Error("ERROR in EZWorker Outer: {0}", ex);
            }
        }
       
    }
}
